(module dirichlet-test mzscheme
  
  (require (planet "test.ss" ("schematics" "schemeunit.plt" 2)))
  (require
   (lib  "comprehensions.ss" "srfi" "42")
   (only (planet "science.ss" ("williams" "science.plt" 2 4)) gamma)
   (file "util.ss")
   (file "dirichlet.ss")
   (file "vector.ss")
   (file "matrix.ss")
   (file "util.ss"))
  
  (provide dirichlet-tests)
  
  (define dirichlet-tests
    (test-suite
     "All tests for dirichlet"

     (test-case
      "Uniform Dirichlet PDF correct"
      (let ([d (v 1.0 1.0)])
        (check-= (dirichlet-pdf (v 0.5 0.5) d) 1.0 0.01)
        (check-= (dirichlet-pdf (v 0.7 0.3)d) 1.0 0.01)))

     (test-case
      "Non-uniform Dirichlet PDF correct"
      (let ([d (v 2.0 2.0)])
        (check-= (dirichlet-pdf (v 0.5 0.5) d) 1.5 0.01)))

     (test-case
      "Dirichlet PDF is zero for point not in simplex"
      (let ([d (v 2.0 2.0)])
        (check-= (dirichlet-pdf (v 0.0 0.0) d) 0.0 0.01)
        (check-= (dirichlet-pdf (v 0.1 0.5) d) 0.0 0.01)
        (check-= (dirichlet-pdf (v 1.0 1.2) d) 0.0 0.01)))

     (test-case
      "dirichlet-pdf raises exn on contract violation"
      (check-exn
       exn:fail:contract?
       (lambda ()
         (dirichlet-pdf (v 0.0 1.0) (v 1.0 1.0 1.0)))))

     (test-case
      "dirichlet-bayesian-update produces correct result"
      (let ([d (v 1.0 1.0)])
        (check-= (dirichlet-pdf (v 0.5 0.5) d) 1.0 0.01)
        (check-=
               (dirichlet-pdf
                (v 0.5 0.5)
                (dirichlet-bayesian-update (v 1.0 1.0) d))
               1.5
               0.01)))

     (test-case
      "dirichlet-bayesian-update raises exn on contract violation"
      (check-exn
       exn:fail:contract?
       (lambda ()
         (let ([d (v 1.0 1.0)])
           (dirichlet-bayesian-update (v 1.0) d)))))

     (test-case
      "dirichlet-pdf is numerically stable"
      (check-false (zero? (dirichlet-pdf (v 0.99 0.005 0.005)
                                         (v 2.0 2.0 2.0)))))

     (test-case
      "random-dirichlet"
      (do-ec
       (:range i 10)
       (check-= (v+/ (random-dirichlet (v 1 2 3)))
                1.0
                0.01)))

     (test-case
      "dirichlet-sample-likelihood correct for simple case"
      (check-= (dirichlet-sample-likelihood (v 1 0) (v 1 1))
               (* (/ (gamma 2) (gamma 3))
                  (* (/ (gamma 2) (gamma 1))
                     (/ (gamma 1) (gamma 1))))
               0.00001))

     (test-case
      "dirichlet-bayesian-retract produces correct result"
      (let* ([d (v 2.0 2.0)]
             [d-prime (dirichlet-bayesian-retract (v 1.0 1.0) d)])
        ;; This NOP prevents a segmentation fault.
        ;; Curiously, other NOPS (e.g. (display "")) have no
        ;; effect.  Who knows what is going on here.
        (v- d-prime d-prime)
        (check-v= d-prime (v 1.0 1.0) 0.0001)
        (check-= (dirichlet-pdf (v 0.5 0.5) d-prime)
                 1.0
                 0.01)))
                                            
     
     ))
  )
