(module dirichlet mzscheme

  (require
   (lib "contract.ss")
   (planet "srfi-4-comprehensions.ss" ("wmfarr" "srfi-4-comprehensions.plt" 1))
   (planet "all.ss" ("wmfarr" "plt-linalg.plt" 1))
   (planet "science.ss" ("williams" "science.plt" 2 4))
   (file "vector.ss")
   (file "matrix.ss"))


  ;; random-dirichlet : vector -> matrix
  (define (random-dirichlet params)
    (let ([samples
           (f64vector-of-length-ec
            (vlength params)
            (:f64vector x params)
            (random-gamma x 1))])
      (v/s samples (v+/ samples))))

  (provide random-dirichlet)
  
  
  (define dirichlet-pdf-contract
    (->r ([point  v?]
          [params (lambda (d)
                    (= (vlength point)
                       (vlength d)))])
         number?))

  ;; dirichlet-pdf : vector vector -> real
  ;;
  ;; Elements of point must be in the range (0,1)
  (define (dirichlet-pdf point params)
    (let ([point-sum (v+/ point)])
      (if (= point-sum 1.0)
          (exp (lndirichlet-pdf point params))
          0.0)))

  (define (lndirichlet-pdf point params)
    (let ([B (- (lngamma (v+/ params))
                (v+/ (vmap lngamma params)))])
      (+ (sum-ec (:f64vector p (index i) point)
                 (:let alpha (sub1 (vref params i)))
                 (* (log p) alpha))
         B)))
  
  ;; dirichlet without the logs
;   (let* ([params (dirichlet-params dirichlet)]
;           [point-sum (sum-ec (:vector p point) p)]
;           [B (/ (product-ec (:vector p params) (gamma p))
;                 (gamma (sum-ec (:vector p params) p)))])
;      (if (= point-sum 1.0)
;          (/ (product-ec (:vector p (index i) point)
;                         (:let alpha (sub1 (vector-ref params i)))
;                         (expt p alpha))
;             B)
;          0.0))
  

  (define (dirichlet-same-dimension-contract return)
    (->r ([samples v?]
          [params  (lambda (d)
                     (= (vlength samples)
                        (vlength d)))])
         return))

  (define (dirichlet-bayesian-update samples params)
    (v+ samples params))

  (define (dirichlet-bayesian-retract samples params)
    (v- params samples))

  ;; dirichlet-sample-likelihood : vector vector -> float
  ;;
  ;; We have a data item, and a Dirichlet distribution.  We
  ;; would like to calculate the likelihood [P(D|H)] of the
  ;; data given the Dirichlet.  This involves integrating
  ;; over all multinomials distributed according to the
  ;; Dirichlet.  There is a standard formula for this,
  ;; e.g. Eqs 49,50 in
  ;;
  ;;  http://research.microsoft.com/~minka/papers/dirichlet/minka-dirichlet.pdf
  ;;
  ;; gamma(sum(k, alpha(k)) / gamma(sum(k, n(k) + alpha(k))) *
  ;;   product(k, gamma(n(k) + alpha(k)) / gamma(alpha(k))
  (define (dirichlet-sample-likelihood samples params)
    (exp
     (+ (- (lngamma (v+/ params))
           (lngamma (sum-ec
                     (:parallel (:f64vector p params)
                                (:f64vector s samples))
                     (+ p s))))
        (sum-ec
         (:parallel (:f64vector p params)
                    (:f64vector s samples))
         (- (lngamma (+ p s))
            (lngamma p))))))
  
  (provide/contract
   [dirichlet-pdf dirichlet-pdf-contract]
   [dirichlet-bayesian-update (dirichlet-same-dimension-contract v?)]
   [dirichlet-bayesian-retract (dirichlet-same-dimension-contract v?)]
   [dirichlet-sample-likelihood (dirichlet-same-dimension-contract number?)])

  )
