(module mixture mzscheme

  (require
   (lib "etc.ss")
   (lib "list.ss")
   (lib "plt-match.ss")
   (file "vector.ss"))
  
  (provide sample-mixture
           display-mixture
           (rename create-mixture make-mixture))

  ;; struct mixture : (f64vector-of real) (vector-of any)
  (define-struct mixture (p v) #f)

  ;; display-mixture : mixture -> void
  (define (display-mixture mixture)
    (let ([p (mixture-p mixture)]
          [v (mixture-v mixture)])
      (display "#mixture(")
      (display-vector p)
      (display " ")
      (display v)
      (display ")")))
  
  ;; make-mixture : (listof (cons real any))
  (define (create-mixture d)
    (let* ([l (length d)]
           [ps (vmake l 0.0)]
           [vs (make-vector l #f)])
      (foldl
       (lambda (component idx)
         (match-let ([`(,p . ,v) component])
           (vset! ps idx p)
           (vector-set! vs idx v)
           (add1 idx)))
       0
       d)
      (make-mixture (vnormalise ps)  vs)))

  ;; sample-mixture : mixture [random-source] -> any
  (define sample-mixture
    (opt-lambda (m [random-source random])
      (let* ([p (random-source)]
             [ps (mixture-p m)]
             [vs (mixture-v m)]
             [sum 1.0]
             [idx (vfoldi
                   (lambda (i x choice)
                     (begin0
                         (if (< p sum)
                             i
                             choice)
                       (set! sum (- sum x))))
                   0
                   ps)])
        (vector-ref vs idx))))
  )
