(module vector-test mzscheme
  
  (require (planet "test.ss" ("schematics" "schemeunit.plt" 2)))
  (require "vector.ss")
  
  (provide vector-tests)
  
  (define vector-tests
    (test-suite
     "All tests for vector"

     (test-case
      "constructor v handles exact and inexact numbers"
      (check v= (v 1 2 3 4) (v 1. 2. 3. 4.)))
     
     (test-case
      "v+/"
      (check = (v+/ (v 1.0 1.0 1.0 1.0 1.0)) 5.0)
      (check = (v+/ (v -1 -2 -3 1 2 3)) 0.0))

     (test-case
      "v+s"
      (check v= (v+s (v 1 2 3 4) 1)  (v 2 3 4 5))
      (check v= (v+s (v 4 3 2 1) -1) (v 3 2 1 0)))
     
     (test-case
      "v-s"
      (check v= (v-s (v 1 2 3 4) 1)  (v 0 1 2 3))
      (check v= (v-s (v 4 3 2 1) -1) (v 5 4 3 2)))

     (test-case
      "vmap"
      (check v= (vmap - (v 1 2 3 4)) (v -1 -2 -3 -4)))

     (test-case
      "vmapi"
      (check v= (vmapi (lambda (i x) i) (v 4 3 2 1)) (v 0 1 2 3)))

     (test-case
      "vfold"
      (check-equal? (vfold (lambda (x seed) (+ x seed)) 4 (v 0 1 2 3))
                    10.0))

     (test-case
      "vfoldi"
      (check-equal? (vfoldi (lambda (i x seed) (+ x i seed)) 4 (v 1 2 3 4))
                    20.0))
     
     (test-case
      "v*."
      (check v= (v*. (v 1 1 1 1) (v 1 1 1 1)) (v 1 1 1 1))
      (check v= (v*. (v 1 2 3 4) (v 4 3 2 1)) (v 4 6 6 4)))

     (test-case
      "v*. checks vectors of same size"
      (check-exn
       exn:fail:contract?
       (lambda ()
         (v*. (v 1 2 3 4) (v 1 2 3 4 5)))))

     (test-case
      "vones"
      (check v= (vones 5) (v 1 1 1 1 1)))

     (test-case
      "vnormalise"
      (check v= (vnormalise (v 1 2 3 4)) (v 0.1 0.2 0.3 0.4))) 
     
     (test-case
      "vmax"
      (check-equal? (vmax (v)) -inf.0)
      (check-= (vmax (v 1 2 3 4)) 4 0.00001)
      (check-= (vmax (v 4 3 2 1)) 4 0.00001)
      (check-= (vmax (v 3 4 2 1)) 4 0.00001))

     (test-case
      "vmax"
      (check-eq? (vmaxi (v)) -1)
      (check-eq? (vmaxi (v 1 2 3 4)) 3)
      (check-eq? (vmaxi (v 4 3 2 1)) 0)
      (check-eq? (vmaxi (v 3 4 2 1)) 1))
     ))
  )
